@@ -89,6 +89,7 @@ HEATMAP_TYPE = 'heatmap'
 STATUSMAP_TYPE = 'flant-statusmap-panel'
 SVG_TYPE = 'marcuscalidus-svg-panel'
 PIE_CHART_TYPE = 'grafana-piechart-panel'
+WORLD_MAP_TYPE = 'grafana-worldmap-panel'
 
 DEFAULT_FILL = 1
 DEFAULT_REFRESH = '10s'
@@ -443,6 +444,14 @@ def is_valid_xaxis_mode(instance, attribute, value):
 
 @attr.s
 class XAxis(object):
+    """
+    X Axis
+
+    :param mode: Mode of axis can be time, series or histogram
+    :param name: X axis name
+    :param value: list of values eg. ["current"] or ["avg"]
+    :param show: show X axis
+    """
 
     mode = attr.ib(default='time', validator=is_valid_xaxis_mode)
     name = attr.ib(default=None)
@@ -451,6 +460,9 @@ class XAxis(object):
 
     def to_json_data(self):
         return {
+            'mode': self.mode,
+            'name': self.name,
+            'values': self.values,
             'show': self.show,
         }
 
@@ -460,7 +472,16 @@ class YAxis(object):
     """A single Y axis.
 
     Grafana graphs have two Y axes: one on the left and one on the right.
+
+    :param decimals: Defines how many decimals are displayed for Y value. (default auto)
+    :param format: The display unit for the Y value
+    :param label: The Y axis label. (default “")
+    :param logBase: The scale to use for the Y value, linear, or logarithmic. (default linear)
+    :param max: The maximum Y value
+    :param min: The minimum Y value
+    :param show: Show or hide the axis
     """
+
     decimals = attr.ib(default=None)
     format = attr.ib(default=None)
     label = attr.ib(default=None)
@@ -756,6 +777,7 @@ class Template(object):
                         break
         else:
             self._current = {
+                'selected': False if self.default is None or not self.default else True,
                 'text': self.default,
                 'value': self.default,
                 'tags': [],
@@ -1119,6 +1141,20 @@ class Dashboard(object):
         }
 
 
+def _deep_update(base_dict, extra_dict):
+    if extra_dict is None:
+        return base_dict
+
+    for k, v in extra_dict.items():
+        if k in base_dict and hasattr(base_dict[k], "to_json_data"):
+            base_dict[k] = base_dict[k].to_json_data()
+
+        if k in base_dict and isinstance(base_dict[k], dict):
+            _deep_update(base_dict[k], v)
+        else:
+            base_dict[k] = v
+
+
 @attr.s
 class Panel(object):
     """
@@ -1142,6 +1178,9 @@ class Panel(object):
     :param timeFrom: time range that Override relative time
     :param title: of the panel
     :param transparent: defines if panel should be transparent
+    :param transformations: defines transformations applied to the table
+    :param extraJson: raw JSON additions or overrides added to the JSON output
+           of this panel, can be used for using unsupported features
     """
 
     dataSource = attr.ib(default=None)
@@ -1164,6 +1203,8 @@ class Panel(object):
     timeFrom = attr.ib(default=None)
     timeShift = attr.ib(default=None)
     transparent = attr.ib(default=False, validator=instance_of(bool))
+    transformations = attr.ib(default=attr.Factory(list), validator=instance_of(list))
+    extraJson = attr.ib(default=None, validator=attr.validators.optional(instance_of(dict)))
 
     def _map_panels(self, f):
         return f(self)
@@ -1192,8 +1233,10 @@ class Panel(object):
             'timeShift': self.timeShift,
             'title': self.title,
             'transparent': self.transparent,
+            'transformations': self.transformations
         }
         res.update(overrides)
+        _deep_update(res, self.extraJson)
         return res
 
 
@@ -1281,19 +1324,30 @@ class Graph(Panel):
     Generates Graph panel json structure.
 
     :param alert: List of AlertConditions
+    :param align: Select to align left and right Y-axes by value
+    :param alignLevel: Available when Align is selected. Value to use for alignment of left and right Y-axes
+    :param bars: Display values as a bar chart
     :param dataLinks: List of data links hooked to datapoints on the graph
-    :param dataSource: DataSource's name
-    :param minSpan: Minimum width for each panel
+    :param fill: Area fill, amount of color fill for a series. (default 1, 0 is none)
+    :param fillGradient: Degree of gradient on the area fill. (0 is no gradient, 10 is a steep gradient. Default is 0.)
+    :param lines: Display values as a line graph
+    :param points: Display points for values (default False)
+    :param pointRadius: Controls how large the points are
+    :param stack: Each series is stacked on top of another
+    :param percentage: Available when Stack is selected. Each series is drawn as a percentage of the total of all series
     :param thresholds: List of GraphThresholds - Only valid when alert not defined
     """
 
     alert = attr.ib(default=None)
     alertThreshold = attr.ib(default=True, validator=instance_of(bool))
     aliasColors = attr.ib(default=attr.Factory(dict))
+    align = attr.ib(default=False, validator=instance_of(bool))
+    alignLevel = attr.ib(default=0, validator=instance_of(int))
     bars = attr.ib(default=False, validator=instance_of(bool))
     dataLinks = attr.ib(default=attr.Factory(list))
     error = attr.ib(default=False, validator=instance_of(bool))
     fill = attr.ib(default=1, validator=instance_of(int))
+    fillGradient = attr.ib(default=0, validator=instance_of(int))
     grid = attr.ib(default=attr.Factory(Grid), validator=instance_of(Grid))
     isNew = attr.ib(default=True, validator=instance_of(bool))
     legend = attr.ib(
@@ -1316,7 +1370,6 @@ class Graph(Panel):
     )
     thresholds = attr.ib(default=attr.Factory(list))
     xAxis = attr.ib(default=attr.Factory(XAxis), validator=instance_of(XAxis))
-    # XXX: This isn't a *good* default, rather it's the default Grafana uses.
     try:
         yAxes = attr.ib(
             default=attr.Factory(YAxes),
@@ -1359,6 +1412,10 @@ class Graph(Panel):
             'type': GRAPH_TYPE,
             'xaxis': self.xAxis,
             'yaxes': self.yAxes,
+            'yaxis': {
+                'align': self.align,
+                'alignLevel': self.alignLevel
+            }
         }
         if self.alert:
             graphObject['alert'] = self.alert
@@ -1506,8 +1563,8 @@ class AlertList(object):
     :param span: Defines the number of spans that will be used for the panel.
     :param stateFilter: Show alerts with statuses from the stateFilter list. The list can contain a
         subset of the following statuses:
-            [ALERTLIST_STATE_ALERTING, ALERTLIST_STATE_OK, ALERTLIST_STATE_NO_DATA,
-             ALERTLIST_STATE_PAUSED, ALERTLIST_STATE_EXECUTION_ERROR, ALERTLIST_STATE_PENDING].
+        [ALERTLIST_STATE_ALERTING, ALERTLIST_STATE_OK, ALERTLIST_STATE_NO_DATA,
+        ALERTLIST_STATE_PAUSED, ALERTLIST_STATE_EXECUTION_ERROR, ALERTLIST_STATE_PENDING].
         An empty list means all alerts.
     :param title: The panel title.
     :param transparent: If true, display the panel without a background.
@@ -1563,43 +1620,31 @@ class Stat(Panel):
 
     Grafana doc on stat: https://grafana.com/docs/grafana/latest/panels/visualizations/stat-panel/
 
-    :param dataSource: Grafana datasource name
-    :param targets: list of metric requests for chosen datasource
-    :param title: panel title
-    :param textMode: define Grafana will show name or value: keys: 'auto' 'name' 'none' 'value' 'value_and_name'
-    :param colorMode: defines if Grafana will color panel background: keys "value" "background"
-    :param graphMode: defines if Grafana will draw graph: keys 'area' 'none'
-    :param orientation: Stacking direction in case of multiple series or fields: keys 'auto' 'horizontal' 'vertical'
     :param alignment: defines value & title positioning: keys 'auto' 'centre'
-    :param description: optional panel description
-    :param editable: defines if panel is editable via web interfaces
-    :param format: defines value units
-    :param height: defines panel height
-    :param id: panel id
+    :param colorMode: defines if Grafana will color panel background: keys "value" "background"
     :param decimals: number of decimals to display
-    :param interval: defines time interval between metric queries
-    :param links: additional web links
+    :param format: defines value units
+    :param graphMode: defines if Grafana will draw graph: keys 'area' 'none'
     :param mappings: the list of values to text mappings
         This should be a list of StatMapping objects
         https://grafana.com/docs/grafana/latest/panels/field-configuration-options/#value-mapping
+    :param orientation: Stacking direction in case of multiple series or fields: keys 'auto' 'horizontal' 'vertical'
     :param reduceCalc: algorithm for reduction to a single value: keys
         'mean' 'lastNotNull' 'last' 'first' 'firstNotNull' 'min' 'max' 'sum' 'total'
-    :param span: defines the number of spans that will be used for panel
+    :param textMode: define Grafana will show name or value: keys: 'auto' 'name' 'none' 'value' 'value_and_name'
     :param thresholds: single stat thresholds
-    :param transparent: defines if the panel should be transparent
     """
 
-    textMode = attr.ib(default='auto')
-    colorMode = attr.ib(default='value')
-    graphMode = attr.ib(default='area')
-    orientation = attr.ib(default='auto')
     alignment = attr.ib(default='auto')
-    format = attr.ib(default='none')
-    mappings = attr.ib(default=attr.Factory(list))
-    span = attr.ib(default=6)
-    thresholds = attr.ib(default="")
-    reduceCalc = attr.ib(default='mean', type=str)
+    colorMode = attr.ib(default='value')
     decimals = attr.ib(default=None)
+    format = attr.ib(default='none')
+    graphMode = attr.ib(default='area')
+    mappings = attr.ib(default=attr.Factory(list))
+    orientation = attr.ib(default='auto')
+    reduceCalc = attr.ib(default='mean', type=str)
+    textMode = attr.ib(default='auto')
+    thresholds = attr.ib(default="")
 
     def to_json_data(self):
         return self.panel_json(
@@ -1718,30 +1763,19 @@ class SingleStat(Panel):
 
     Grafana doc on singlestat: https://grafana.com/docs/grafana/latest/features/panels/singlestat/
 
-    :param dataSource: Grafana datasource name
-    :param targets: list of metric requests for chosen datasource
-    :param title: panel title
     :param cacheTimeout: metric query result cache ttl
     :param colors: the list of colors that can be used for coloring
         panel value or background. Additional info on coloring in docs:
         https://grafana.com/docs/grafana/latest/features/panels/singlestat/#coloring
     :param colorBackground: defines if grafana will color panel background
     :param colorValue: defines if grafana will color panel value
-    :param description: optional panel description
     :param decimals: override automatic decimal precision for legend/tooltips
-    :param editable: defines if panel is editable via web interfaces
     :param format: defines value units
     :param gauge: draws and additional speedometer-like gauge based
-    :param height: defines panel height
-    :param hideTimeOverride: hides time overrides
-    :param id: panel id
-    :param interval: defines time interval between metric queries
-    :param links: additional web links
     :param mappingType: defines panel mapping type.
         Additional info can be found in docs:
         https://grafana.com/docs/grafana/latest/features/panels/singlestat/#value-to-text-mapping
     :param mappingTypes: the list of available mapping types for panel
-    :param minSpan: minimum span number
     :param nullText: defines what to show if metric query result is undefined
     :param nullPointMode: defines how to render undefined values
     :param postfix: defines postfix that will be attached to value
@@ -1749,17 +1783,14 @@ class SingleStat(Panel):
     :param prefix: defines prefix that will be attached to value
     :param prefixFontSize: defines prefix font size
     :param rangeMaps: the list of value to text mappings
-    :param span: defines the number of spans that will be used for panel
     :param sparkline: defines if grafana should draw an additional sparkline.
         Sparkline grafana documentation:
         https://grafana.com/docs/grafana/latest/features/panels/singlestat/#spark-lines
     :param thresholds: single stat thresholds
-    :param transparent: defines if panel should be transparent
     :param valueFontSize: defines value font size
     :param valueName: defines value type. possible values are:
         min, max, avg, current, total, name, first, delta, range
     :param valueMaps: the list of value to text mappings
-    :param timeFrom: time range that Override relative time
     """
 
     cacheTimeout = attr.ib(default=None)
@@ -1770,8 +1801,6 @@ class SingleStat(Panel):
     format = attr.ib(default='none')
     gauge = attr.ib(default=attr.Factory(Gauge),
                     validator=instance_of(Gauge))
-    hideTimeOverride = attr.ib(default=False, validator=instance_of(bool))
-    interval = attr.ib(default=None)
     mappingType = attr.ib(default=MAPPING_TYPE_VALUE_TO_TEXT)
     mappingTypes = attr.ib(
         default=attr.Factory(lambda: [
@@ -1806,8 +1835,6 @@ class SingleStat(Panel):
                 'decimals': self.decimals,
                 'format': self.format,
                 'gauge': self.gauge,
-                'interval': self.interval,
-                'hideTimeOverride': self.hideTimeOverride,
                 'mappingType': self.mappingType,
                 'mappingTypes': self.mappingTypes,
                 'minSpan': self.minSpan,
@@ -2001,31 +2028,17 @@ class Table(Panel):
     Grafana doc on table: https://grafana.com/docs/grafana/latest/features/panels/table_panel/#table-panel
 
     :param columns: table columns for Aggregations view
-    :param dataSource: Grafana datasource name
-    :param description: optional panel description
-    :param editable: defines if panel is editable via web interfaces
     :param fontSize: defines value font size
-    :param height: defines panel height
-    :param hideTimeOverride: hides time overrides
-    :param id: panel id
-    :param links: additional web links
-    :param minSpan: minimum span number
     :param pageSize: rows per page (None is unlimited)
     :param scroll: scroll the table instead of displaying in full
     :param showHeader: show the table header
-    :param span: defines the number of spans that will be used for panel
+    :param sort: table sorting
     :param styles: defines formatting for each column
-    :param targets: list of metric requests for chosen datasource
-    :param timeFrom: time range that Override relative time
-    :param title: panel title
     :param transform: table style
-    :param transparent: defines if panel should be transparent
     """
 
     columns = attr.ib(default=attr.Factory(list))
     fontSize = attr.ib(default='100%')
-    hideTimeOverride = attr.ib(default=False, validator=instance_of(bool))
-    minSpan = attr.ib(default=None)
     pageSize = attr.ib(default=None)
     scroll = attr.ib(default=True, validator=instance_of(bool))
     showHeader = attr.ib(default=True, validator=instance_of(bool))
@@ -2094,41 +2107,25 @@ class BarGauge(Panel):
     """Generates Bar Gauge panel json structure
 
     :param allValue: If All values should be shown or a Calculation
-    :param cacheTimeout: metric query result cache ttl
     :param calc: Calculation to perform on metrics
     :param dataLinks: list of data links hooked to datapoints on the graph
-    :param dataSource: Grafana datasource name
     :param decimals: override automatic decimal precision for legend/tooltips
-    :param description: optional panel description
     :param displayMode: style to display bar gauge in
-    :param editable: defines if panel is editable via web interfaces
     :param format: defines value units
-    :param height: defines panel height
-    :param hideTimeOverride: hides time overrides
-    :param id: panel id
-    :param interval: defines time interval between metric queries
     :param labels: option to show gauge level labels
     :param limit: limit of number of values to show when not Calculating
-    :param links: additional web links
     :param max: maximum value of the gauge
     :param min: minimum value of the gauge
-    :param minSpan: minimum span number
     :param orientation: orientation of the bar gauge
     :param rangeMaps: the list of value to text mappings
-    :param span: defines the number of spans that will be used for panel
-    :param targets: list of metric requests for chosen datasource
     :param thresholdLabel: label for gauge. Template Variables:
         "$__series_namei" "$__field_name" "$__cell_{N} / $__calc"
     :param thresholdMarkers: option to show marker of level on gauge
     :param thresholds: single stat thresholds
-    :param timeFrom: time range that Override relative time
-    :param title: panel title
-    :param transparent: defines if panel should be transparent
     :param valueMaps: the list of value to text mappings
     """
 
     allValues = attr.ib(default=False, validator=instance_of(bool))
-    cacheTimeout = attr.ib(default=None)
     calc = attr.ib(default=GAUGE_CALC_MEAN)
     dataLinks = attr.ib(default=attr.Factory(list))
     decimals = attr.ib(default=None)
@@ -2143,13 +2140,10 @@ class BarGauge(Panel):
         ),
     )
     format = attr.ib(default='none')
-    hideTimeOverride = attr.ib(default=False, validator=instance_of(bool))
-    interval = attr.ib(default=None)
     label = attr.ib(default=None)
     limit = attr.ib(default=None)
     max = attr.ib(default=100)
     min = attr.ib(default=0)
-    minSpan = attr.ib(default=None)
     orientation = attr.ib(
         default=ORIENTATION_HORIZONTAL,
         validator=in_([ORIENTATION_HORIZONTAL, ORIENTATION_VERTICAL]),
@@ -2171,10 +2165,6 @@ class BarGauge(Panel):
     def to_json_data(self):
         return self.panel_json(
             {
-                'cacheTimeout': self.cacheTimeout,
-                'hideTimeOverride': self.hideTimeOverride,
-                'interval': self.interval,
-                'minSpan': self.minSpan,
                 'options': {
                     'displayMode': self.displayMode,
                     'fieldOptions': {
@@ -2207,50 +2197,31 @@ class GaugePanel(Panel):
     """Generates Gauge panel json structure
 
     :param allValue: If All values should be shown or a Calculation
-    :param cacheTimeout: metric query result cache ttl
     :param calc: Calculation to perform on metrics
     :param dataLinks: list of data links hooked to datapoints on the graph
-    :param dataSource: Grafana datasource name
     :param decimals: override automatic decimal precision for legend/tooltips
-    :param description: optional panel description
-    :param editable: defines if panel is editable via web interfaces
     :param format: defines value units
-    :param height: defines panel height
-    :param hideTimeOverride: hides time overrides
-    :param id: panel id
-    :param interval: defines time interval between metric queries
     :param labels: option to show gauge level labels
     :param limit: limit of number of values to show when not Calculating
-    :param links: additional web links
     :param max: maximum value of the gauge
     :param min: minimum value of the gauge
-    :param minSpan: minimum span number
     :param rangeMaps: the list of value to text mappings
-    :param span: defines the number of spans that will be used for panel
-    :param targets: list of metric requests for chosen datasource
     :param thresholdLabel: label for gauge. Template Variables:
         "$__series_namei" "$__field_name" "$__cell_{N} / $__calc"
     :param thresholdMarkers: option to show marker of level on gauge
     :param thresholds: single stat thresholds
-    :param timeFrom: time range that Override relative time
-    :param title: panel title
-    :param transparent: defines if panel should be transparent
     :param valueMaps: the list of value to text mappings
     """
 
     allValues = attr.ib(default=False, validator=instance_of(bool))
-    cacheTimeout = attr.ib(default=None)
     calc = attr.ib(default=GAUGE_CALC_MEAN)
     dataLinks = attr.ib(default=attr.Factory(list))
     decimals = attr.ib(default=None)
     format = attr.ib(default='none')
-    hideTimeOverride = attr.ib(default=False, validator=instance_of(bool))
-    interval = attr.ib(default=None)
     label = attr.ib(default=None)
     limit = attr.ib(default=None)
     max = attr.ib(default=100)
     min = attr.ib(default=0)
-    minSpan = attr.ib(default=None)
     rangeMaps = attr.ib(default=attr.Factory(list))
     thresholdLabels = attr.ib(default=False, validator=instance_of(bool))
     thresholdMarkers = attr.ib(default=True, validator=instance_of(bool))
@@ -2268,10 +2239,6 @@ class GaugePanel(Panel):
     def to_json_data(self):
         return self.panel_json(
             {
-                'cacheTimeout': self.cacheTimeout,
-                'hideTimeOverride': self.hideTimeOverride,
-                'interval': self.interval,
-                'minSpan': self.minSpan,
                 'options': {
                     'fieldOptions': {
                         'calcs': [self.calc],
@@ -2301,13 +2268,13 @@ class GaugePanel(Panel):
 class HeatmapColor(object):
     """A Color object for heatmaps
 
-    :param cardColor
-    :param colorScale
-    :param colorScheme
-    :param exponent
-    :param max
-    :param min
-    :param mode
+    :param cardColor: color
+    :param colorScale: scale
+    :param colorScheme: scheme
+    :param exponent: exponent
+    :param max: max
+    :param min: min
+    :param mode: mode
     """
 
     # Maybe cardColor should validate to RGBA object, not sure
@@ -2335,16 +2302,16 @@ class HeatmapColor(object):
 class Heatmap(Panel):
     """Generates Heatmap panel json structure (https://grafana.com/docs/grafana/latest/features/panels/heatmap/)
 
-    :param heatmap
+    :param heatmap: dict
     :param cards: A heatmap card object: keys "cardPadding", "cardRound"
     :param color: Heatmap color object
     :param dataFormat: 'timeseries' or 'tsbuckets'
     :param yBucketBound: 'auto', 'upper', 'middle', 'lower'
     :param reverseYBuckets: boolean
-    :param xBucketSize
-    :param xBucketNumber
-    :param yBucketSize
-    :param yBucketNumber
+    :param xBucketSize: Size
+    :param xBucketNumber: Number
+    :param yBucketSize: Size
+    :param yBucketNumber: Number
     :param highlightCards: boolean
     :param hideZeroBuckets: boolean
     :param transparent: defines if the panel should be transparent
@@ -2419,14 +2386,14 @@ class Heatmap(Panel):
 class StatusmapColor(object):
     """A Color object for Statusmaps
 
-    :param cardColor
-    :param colorScale
-    :param colorScheme
-    :param exponent
-    :param max
-    :param min
-    :param mode
-    :param thresholds
+    :param cardColor: colour
+    :param colorScale: scale
+    :param colorScheme: scheme
+    :param exponent: exponent
+    :param max: max
+    :param min: min
+    :param mode: mode
+    :param thresholds: threshold
     """
 
     # Maybe cardColor should validate to RGBA object, not sure
@@ -2457,27 +2424,15 @@ class Statusmap(Panel):
     """Generates json structure for the flant-statusmap-panel visualisation plugin
     (https://grafana.com/grafana/plugins/flant-statusmap-panel/).
 
-    :param alert
+    :param alert: Alert
     :param cards: A statusmap card object: keys 'cardRound', 'cardMinWidth', 'cardHSpacing', 'cardVSpacing'
     :param color: A StatusmapColor object
-    :param dataSource: Name of the datasource to use
-    :param description: Description of the panel
-    :param editable
-    :param id
-    :param isNew
-    :param legend
-    :param links
-    :param minSpan
-    :param nullPointMode
-    :param span
-    :param targets
-    :param timeFrom
-    :param timeShift
-    :param title: Title of the panel
-    :param tooltip
-    :param transparent: Set panel transparency on/off
-    :param xAxis
-    :param yAxis
+    :param isNew: isNew
+    :param legend: Legend object
+    :param nullPointMode: null
+    :param tooltip: Tooltip object
+    :param xAxis: XAxis object
+    :param yAxis: YAxis object
     """
 
     alert = attr.ib(default=None)
@@ -2499,7 +2454,6 @@ class Statusmap(Panel):
         default=attr.Factory(Legend),
         validator=instance_of(Legend),
     )
-    minSpan = attr.ib(default=None)
     nullPointMode = attr.ib(default=NULL_AS_ZERO)
     tooltip = attr.ib(
         default=attr.Factory(Tooltip),
@@ -2535,21 +2489,12 @@ class Statusmap(Panel):
 class Svg(Panel):
     """Generates SVG panel json structure
     Grafana doc on SVG: https://grafana.com/grafana/plugins/marcuscalidus-svg-panel
-    :param dataSource: Grafana datasource name
-    :param targets: list of metric requests for chosen datasource
-    :param title: panel title
-    :param description: optional panel description
-    :param editable: defines if panel is editable via web interfaces
+
     :param format: defines value units
     :param jsCodeFilePath: path to javascript file to be run on dashboard refresh
     :param jsCodeInitFilePath: path to javascript file to be run after the first initialization of the SVG
-    :param height: defines panel height
-    :param id: panel id
-    :param interval: defines time interval between metric queries
-    :param links: additional web links
-    :param reduceCalc: algorithm for reduction to a single value:
+    :param reduceCalc: algorithm for reduction to a single value,
         keys 'mean' 'lastNotNull' 'last' 'first' 'firstNotNull' 'min' 'max' 'sum' 'total'
-    :param span: defines the number of spans that will be used for panel
     :param svgFilePath: path to SVG image file to be displayed
     """
 
@@ -2590,33 +2535,32 @@ class Svg(Panel):
 class PieChart(Panel):
     """Generates Pie Chart panel json structure
     Grafana doc on Pie Chart: https://grafana.com/grafana/plugins/grafana-piechart-panel
-    :param dataSource: Grafana datasource name
-    :param targets: list of metric requests for chosen datasource
-    :param title: panel title
-    :param description: optional panel description
-    :param editable: defines if panel is editable via web interfaces
+
+    :param aliasColors: dictionary of color overrides
     :param format: defines value units
-    :param height: defines panel height
-    :param id: panel id
     :param pieType: defines the shape of the pie chart (pie or donut)
+    :param percentageDecimals: Number of decimal places to show if percentages shown in legned
     :param showLegend: defines if the legend should be shown
     :param showLegendValues: defines if the legend should show values
+    :param showLegendPercentage: Show percentages in the legend
     :param legendType: defines where the legend position
-    :param links: additional web links
-    :param span: defines the number of spans that will be used for panel
-    :param transparent: defines if the panel is transparent
+    :param thresholds: defines thresholds
     """
 
+    aliasColors = attr.ib(default=attr.Factory(dict))
     format = attr.ib(default='none')
     legendType = attr.ib(default='Right side')
     pieType = attr.ib(default='pie')
+    percentageDecimals = attr.ib(default=0, validator=instance_of(int))
     showLegend = attr.ib(default=True)
     showLegendValues = attr.ib(default=True)
+    showLegendPercentage = attr.ib(default=False, validator=instance_of(bool))
     thresholds = attr.ib(default="")
 
     def to_json_data(self):
         return self.panel_json(
             {
+                'aliasColors': self.aliasColors,
                 'format': self.format,
                 'pieType': self.pieType,
                 'height': self.height,
@@ -2628,7 +2572,9 @@ class PieChart(Panel):
                 },
                 'legend': {
                     'show': self.showLegend,
-                    'values': self.showLegendValues
+                    'values': self.showLegendValues,
+                    'percentage': self.showLegendPercentage,
+                    'percentageDecimals': self.percentageDecimals
                 },
                 'legendType': self.legendType,
                 'type': PIE_CHART_TYPE,
@@ -2640,14 +2586,6 @@ class PieChart(Panel):
 class DashboardList(Panel):
     """Generates Dashboard list panel json structure
     Grafana doc on Dashboard list: https://grafana.com/docs/grafana/latest/panels/visualizations/dashboard-list-panel/
-    :param title: panel title
-    :param description: optional panel description
-    :param editable: defines if panel is editable via web interfaces
-    :param height: defines panel height
-    :param id: panel id
-    :param links: additional web links
-    :param span: defines the number of spans that will be used for panel
-    :param transparent: defines if the panel is transparent
 
     :param showHeadings: The chosen list selection (Starred, Recently viewed, Search) is shown as a heading
     :param showSearch: Display dashboards by search query or tags.
@@ -2691,14 +2629,6 @@ class DashboardList(Panel):
 class Logs(Panel):
     """Generates Logs panel json structure
     Grafana doc on Logs panel: https://grafana.com/docs/grafana/latest/panels/visualizations/logs-panel/
-    :param title: panel title
-    :param description: optional panel description
-    :param editable: defines if panel is editable via web interfaces
-    :param height: defines panel height
-    :param id: panel id
-    :param links: additional web links
-    :param span: defines the number of spans that will be used for panel
-    :param transparent: defines if the panel is transparent
 
     :param showLabels: Show or hide the unique labels column, which shows only non-common labels
     :param showTime: Show or hide the log timestamp column
@@ -2749,8 +2679,8 @@ class Threshold(object):
     Example::
         thresholds = [
             Threshold('green', 0, 0.0),
-            Threshold('red', 1, 80.0)
-        ]
+            Threshold('red', 1, 80.0)]
+
     """
 
     color = attr.ib()
@@ -2789,8 +2719,9 @@ class GraphThreshold(object):
     Example:
         thresholds = [
             GraphThreshold(colorMode="ok", value=10.0),
-            GrpahThreshold(colorMode="critical", value=90.0)
-        ]
+            GraphThreshold(colorMode="critical", value=90.0)
+            ]
+
     """
 
     value = attr.ib(validator=instance_of(float))
@@ -2835,3 +2766,99 @@ class SeriesOverride(object):
             'yaxis': self.yaxis,
             'color': self.color,
         }
+
+
+WORLDMAP_CENTER = ['(0°, 0°)', 'North America', 'Europe', 'West Asia', 'SE Asia', 'Last GeoHash', 'custom']
+WORLDMAP_LOCATION_DATA = ['countries', 'countries_3letter', 'states', 'probes', 'geohash', 'json_endpoint', 'jsonp endpoint', 'json result', 'table']
+
+
+@attr.s
+class Worldmap(Panel):
+    """Generates Worldmap panel json structure
+    Grafana doc on Worldmap: https://grafana.com/grafana/plugins/grafana-worldmap-panel/
+
+    :param aggregation: metric aggregation: min, max, avg, current, total
+    :param circleMaxSize: Maximum map circle size
+    :param circleMinSize: Minimum map circle size
+    :param decimals: Number of decimals to show
+    :param geoPoint: Name of the geo_point/geohash column. This is used to calculate where the circle should be drawn.
+    :param locationData: Format of the location data, options in `WORLDMAP_LOCATION_DATA`
+    :param locationName: Name of the Location Name column. Used to label each circle on the map. If it is empty then the geohash value is used.
+    :param metric: Name of the metric column. This is used to give the circle a value - this determines how large the circle is.
+    :param mapCenter: Where to centre the map, default center (0°, 0°). Options: North America, Europe, West Asia, SE Asia, Last GeoHash, custom
+    :param mapCenterLatitude: If mapCenter=custom set the initial map latitude
+    :param mapCenterLongitude: If mapCenter=custom set the initial map longitude
+    :param hideEmpty: Hide series with only nulls
+    :param hideZero: Hide series with only zeros
+    :param initialZoom: Initial map zoom
+    :param jsonUrl: URL for JSON location data if `json_endpoint` or `jsonp endpoint` used
+    :param jsonpCallback: Callback if `jsonp endpoint` used
+    :param mouseWheelZoom: Zoom map on scroll of mouse wheel
+    :param stickyLabels: Sticky map labels
+    :param thresholds: String of thresholds eg. '0,10,20'
+    :param thresholdsColors: List of colors to be used in each threshold
+    :param unitPlural: Units plural
+    :param unitSingle: Units single
+    :param unitSingular: Units singular
+    """
+
+    circleMaxSize = attr.ib(default=30, validator=instance_of(int))
+    circleMinSize = attr.ib(default=2, validator=instance_of(int))
+    decimals = attr.ib(default=0, validator=instance_of(int))
+    geoPoint = attr.ib(default='geohash', validator=instance_of(str))
+    locationData = attr.ib(default='countries', validator=attr.validators.in_(WORLDMAP_LOCATION_DATA))
+    locationName = attr.ib(default='')
+    hideEmpty = attr.ib(default=False, validator=instance_of(bool))
+    hideZero = attr.ib(default=False, validator=instance_of(bool))
+    initialZoom = attr.ib(default=1, validator=instance_of(int))
+    jsonUrl = attr.ib(default='', validator=instance_of(str))
+    jsonpCallback = attr.ib(default='', validator=instance_of(str))
+    mapCenter = attr.ib(default='(0°, 0°)', validator=attr.validators.in_(WORLDMAP_CENTER))
+    mapCenterLatitude = attr.ib(default=0, validator=instance_of(int))
+    mapCenterLongitude = attr.ib(default=0, validator=instance_of(int))
+    metric = attr.ib(default='Value')
+    mouseWheelZoom = attr.ib(default=False, validator=instance_of(bool))
+    stickyLabels = attr.ib(default=False, validator=instance_of(bool))
+    thresholds = attr.ib(default='0,100,150', validator=instance_of(str))
+    thresholdColors = attr.ib(default=["#73BF69", "#73BF69", "#FADE2A", "#C4162A"], validator=instance_of(list))
+    unitPlural = attr.ib(default='', validator=instance_of(str))
+    unitSingle = attr.ib(default='', validator=instance_of(str))
+    unitSingular = attr.ib(default='', validator=instance_of(str))
+    aggregation = attr.ib(default='total', validator=instance_of(str))
+
+    def to_json_data(self):
+        return self.panel_json(
+            {
+                'circleMaxSize': self.circleMaxSize,
+                'circleMinSize': self.circleMinSize,
+                'colors': self.thresholdColors,
+                'decimals': self.decimals,
+                'esGeoPoint': self.geoPoint,
+                'esMetric': self.metric,
+                'locationData': self.locationData,
+                'esLocationName': self.locationName,
+                'hideEmpty': self.hideEmpty,
+                'hideZero': self.hideZero,
+                'initialZoom': self.initialZoom,
+                'jsonUrl': self.jsonUrl,
+                'jsonpCallback': self.jsonpCallback,
+                'mapCenter': self.mapCenter,
+                'mapCenterLatitude': self.mapCenterLatitude,
+                'mapCenterLongitude': self.mapCenterLongitude,
+                'mouseWheelZoom': self.mouseWheelZoom,
+                'stickyLabels': self.stickyLabels,
+                'thresholds': self.thresholds,
+                'unitPlural': self.unitPlural,
+                'unitSingle': self.unitSingle,
+                'unitSingular': self.unitSingular,
+                'valueName': self.aggregation,
+                'tableQueryOptions': {
+                    'queryType': 'geohash',
+                    'geohashField': 'geohash',
+                    'latitudeField': 'latitude',
+                    'longitudeField': 'longitude',
+                    'metricField': 'metric'
+                },
+                'type': WORLD_MAP_TYPE
+            }
+        )
